# ------------------------------------------------------------------------------------- #
# --------------------------------- Appearance Models --------------------------------- #
# ------------------------------------------------------------------------------------- #
AM_INCLUDE_DIR = AM/include
AM_SRC_DIR = AM/src
AM_HEADER_DIR = ${AM_INCLUDE_DIR}/mtf/AM
AM_BASE_HEADERS =  ${AM_HEADER_DIR}/AppearanceModel.h  ${AM_HEADER_DIR}/ImageBase.h

MTF_INCLUDE_DIRS += ${AM_INCLUDE_DIR}

APPEARANCE_MODELS = ImageBase SSDBase SSD NSSD ZNCC SCV LSCV RSCV LRSCV CCRE LKLD MI SSIM NCC SPSS KLD  
APPEARANCE_OBJS = $(addprefix ${BUILD_DIR}/,$(addsuffix .o, ${APPEARANCE_MODELS}))
APPEARANCE_HEADERS = $(addprefix ${AM_HEADER_DIR}/, $(addsuffix .h, ${APPEARANCE_MODELS}))
APPEARANCE_HEADERS += ${AM_BASE_HEADERS}

MTF_HEADERS += ${APPEARANCE_HEADERS}
MTF_OBJS += ${APPEARANCE_OBJS}

MI_FLAGS = 
CCRE_FLAGS = 
LSCV_FLAGS = 
SSD_FLAGS = 

fmaps ?= 0
pca ?= 1
mil ?= 0
mid ?= 0
mitbb ?= 0
miomp ?= 0
ccretbb ?= 0
ccreomp ?= 0
sg ?= 0
lscd ?= 0


ifeq (${fmaps}, 1)
APPEARANCE_MODELS += FMaps
FLAGS64 += -I/usr/lib/caffe_fcn/include/ -I/usr/local/cuda-7.5/targets/x86_64-linux/include/ -I/usr/lib/caffe_fcn/.build_release/include/
MTF_LIBS += -L/usr/lib/caffe_fcn/.build_release/lib -lcaffe
else
CT_FLAGS += -DDISABLE_FMAPS
MTF_RUNTIME_FLAGS += -DDISABLE_FMAPS
endif
ifeq (${pca}, 1)
APPEARANCE_MODELS += PCA
else
CT_FLAGS += -DDISABLE_PCA
MTF_RUNTIME_FLAGS += -DDISABLE_PCA
endif

ifeq (${mitbb}, 1)
MI_FLAGS += -D ENABLE_TBB
MTF_RUNTIME_FLAGS += -D ENABLE_PARALLEL
endif

ifeq (${miomp}, 1)
MI_FLAGS += -D ENABLE_OMP -fopenmp
MTF_RUNTIME_FLAGS += -D ENABLE_PARALLEL -fopenmp
MTF_LIBS += -fopenmp
endif

ifeq (${ccretbb}, 1)
CCRE_FLAGS += -D ENABLE_TBB
MTF_RUNTIME_FLAGS += -D ENABLE_PARALLEL
endif

ifeq (${ccreomp}, 1)
CCRE_FLAGS += -D ENABLE_OMP -fopenmp
MTF_RUNTIME_FLAGS += -D ENABLE_PARALLEL -fopenmp
MTF_LIBS += -fopenmp
endif

ifeq (${mid}, 1)
MI_FLAGS = -D LOG_MI_DATA
endif

ifeq (${mil}, 1)
MI_FLAGS = -D LOG_MI_DATA -D LOG_MI_TIMES 
endif

ifeq (${sg}, 1)
SSD_FLAGS += -D USE_SLOW_GRAD
endif

ifeq (${lscd}, 1)
LSCV_FLAGS += -D LOG_LSCV_DATA
endif
${BUILD_DIR}/ImageBase.o: ${AM_SRC_DIR}/ImageBase.cc  ${AM_HEADER_DIR}/ImageBase.h ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSD_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/SSDBase.o: ${AM_SRC_DIR}/SSDBase.cc ${AM_HEADER_DIR}/SSDBase.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSD_FLAGS} $< ${FLAGS64} -o $@	
	
${BUILD_DIR}/SSD.o: ${AM_SRC_DIR}/SSD.cc ${AM_HEADER_DIR}/SSD.h ${AM_HEADER_DIR}/SSDBase.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSD_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/NSSD.o: ${AM_SRC_DIR}/NSSD.cc ${AM_HEADER_DIR}/NSSD.h ${AM_HEADER_DIR}/SSDBase.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSD_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/ZNCC.o: ${AM_SRC_DIR}/ZNCC.cc ${AM_HEADER_DIR}/ZNCC.h ${AM_HEADER_DIR}/SSDBase.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSD_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/SCV.o: ${AM_SRC_DIR}/SCV.cc ${AM_HEADER_DIR}/SCV.h ${AM_HEADER_DIR}/SSDBase.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSD_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/LSCV.o: ${AM_SRC_DIR}/LSCV.cc ${AM_HEADER_DIR}/LSCV.h ${AM_HEADER_DIR}/SSDBase.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSD_FLAGS} ${LSCV_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/RSCV.o: ${AM_SRC_DIR}/RSCV.cc ${AM_HEADER_DIR}/RSCV.h ${AM_HEADER_DIR}/SSDBase.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSD_FLAGS} $< ${FLAGS64} -o $@	
	
${BUILD_DIR}/LRSCV.o: ${AM_SRC_DIR}/LRSCV.cc ${AM_HEADER_DIR}/LRSCV.h ${AM_HEADER_DIR}/SSDBase.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSD_FLAGS} ${LRSCV_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/FMaps.o: ${AM_SRC_DIR}/FMaps.cc ${AM_HEADER_DIR}/FMaps.h ${AM_HEADER_DIR}/SSDBase.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSD_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/PCA.o: ${AM_SRC_DIR}/PCA.cc ${AM_HEADER_DIR}/PCA.h  ${AM_HEADER_DIR}/SSDBase.h ${AM_BASE_HEADERS} ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${PCA_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/KLD.o: ${AM_SRC_DIR}/KLD.cc ${AM_HEADER_DIR}/KLD.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/histUtils.h ${UTILITIES_HEADER_DIR}/imgUtils.h ${UTILITIES_HEADER_DIR}/miscUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${KLD_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/LKLD.o: ${AM_SRC_DIR}/LKLD.cc ${AM_HEADER_DIR}/LKLD.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/histUtils.h ${UTILITIES_HEADER_DIR}/imgUtils.h ${UTILITIES_HEADER_DIR}/miscUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${LKLD_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/MI.o: ${AM_SRC_DIR}/MI.cc ${AM_HEADER_DIR}/MI.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/histUtils.h ${UTILITIES_HEADER_DIR}/imgUtils.h ${UTILITIES_HEADER_DIR}/miscUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${MI_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/CCRE.o: ${AM_SRC_DIR}/CCRE.cc ${AM_HEADER_DIR}/CCRE.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/histUtils.h ${UTILITIES_HEADER_DIR}/imgUtils.h ${UTILITIES_HEADER_DIR}/miscUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${CCRE_FLAGS} $< ${FLAGS64} -o $@	

${BUILD_DIR}/SSIM.o: ${AM_SRC_DIR}/SSIM.cc ${AM_HEADER_DIR}/SSIM.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SSIM_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/SPSS.o: ${AM_SRC_DIR}/SPSS.cc ${AM_HEADER_DIR}/SPSS.h ${AM_BASE_HEADERS}  ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${SPSS_FLAGS} $< ${FLAGS64} -o $@
	
${BUILD_DIR}/NCC.o: ${AM_SRC_DIR}/NCC.cc ${AM_HEADER_DIR}/NCC.h ${AM_BASE_HEADERS} ${MACROS_HEADER_DIR}/common.h ${UTILITIES_HEADER_DIR}/imgUtils.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${NCC_FLAGS} $< ${FLAGS64} -o $@	

	
