#include "mtf/AM/FMaps.h"
#include <opencv2/core/eigen.hpp>
#include "mtf/Utilities/imgUtils.h"

_MTF_BEGIN_NAMESPACE

cv::Mat FMaps::convert_float_img(Mat &img)
{
    Mat imgU;
    double minVal; 
    double maxVal; 
    Point minLoc; 
    Point maxLoc;
    minMaxLoc(img, &minVal, &maxVal, &minLoc, &maxLoc);
    img -= minVal;
    img.convertTo(imgU,CV_8U,255.0/(maxVal-minVal));
    return imgU;
}

void FMaps::initializePixVals(const Matrix2Xd& init_pts)
{
    n_pix_temp= resx*resy;
	if(!isInitialized()->pix_vals){
		init_pix_vals_temp.resize(n_pix_temp);
		curr_pix_vals_temp.resize(n_pix_temp);
	}
	++frame_count;

	utils::getPixVals(init_pix_vals_temp, curr_img, init_pts, n_pix_temp,
		img_height, img_width, pix_norm_mult, pix_norm_add);
    
    //Create the current patch
	if(!isInitialized()->pix_vals){
		curr_pix_vals_temp = init_pix_vals_temp;
		isInitialized()->pix_vals = true;
	}
    //std::cout<<"starting a new patch"<<std::endl;
    cv::Mat init_patch(resx, resy, CV_8U);
    for(int i=0; i<resx; i++)
    {
        uchar *row= init_patch.ptr(i);
        for(int j=0; j<resy; j++)
        {
            row[j]= init_pix_vals_temp(i*resx+j);
        }
    }
    //imshow("testing patch", init_patch);
    //waitKey();
    //std::cout<<"Samplong Resolution "<<resx<<" "<<resy<<" n_pix "<<n_pix<<std::endl;
    
    //Extract feature maps of this patch
    init_fmaps= extract_features(init_patch, layer_name);
    
    //std::cout<<"size of feature maps "<<init_fmaps.size()<<" "<<init_fmaps[0].rows<<" "<<init_fmaps[0].cols<<" type "<<init_fmaps[0].type()<<std::endl;
    //n_pix= nfmaps*init_fmaps[0].rows*init_fmaps[0].cols;
    init_pix_vals.resize(n_pix);
	curr_pix_vals.resize(n_pix);

    temp_fmaps.resize(init_fmaps[0].rows*init_fmaps[0].cols);
    int c=0;
    int c1=0;
    for (int m=1; m<nfmaps+1; m++)	
    {
        for(int i=0; i<init_fmaps[m].rows; i++)
        {
            float *row= init_fmaps[m].ptr<float>(i);
            for( int j=0; j<init_fmaps[m].cols; j++)
            {
                temp_fmaps(c1)= row[j];
                c1++;
            }
        
        }
        
        if(e_zncc) 
        {
            init_pix_mean = temp_fmaps.mean();
        	temp_fmaps = (temp_fmaps.array() - init_pix_mean);
            init_pix_var = temp_fmaps.squaredNorm() / (init_fmaps[0].rows*init_fmaps[0].cols);
            init_pix_std = sqrt(init_pix_var);
            temp_fmaps /= init_pix_std;
            
            //pix_norm_mult = 1.0 / curr_pix_std;
            //pix_norm_add = curr_pix_mean;
        }   
        c1=0;
        for(int i=0; i<init_fmaps[m].rows; i++)
        {
            for( int j=0; j<init_fmaps[m].cols; j++)
            {
                init_pix_vals(c)= temp_fmaps(c1);
                c++;
                c1++;
            }
        }
        c1=0;
    }


    curr_pix_vals = init_pix_vals;
    
    std::cout<<"The number of feature maps set is "<<nfmaps<<std::endl;
    std::cout<<"Layer name is "<<layer_name<<std::endl;
    std::cout<<"visualizing is "<<vis<<std::endl;
    std::cout<<"The ZNCC option is  new"<<e_zncc<<std::endl;
}

void FMaps::updatePixVals(const Matrix2Xd& curr_pts){
	
    utils::getPixVals(curr_pix_vals_temp, curr_img, curr_pts, n_pix_temp, img_height, img_width, 
		pix_norm_mult, pix_norm_add);
    //std::cout<<"starting a new patch"<<std::endl;
    cv::Mat curr_patch(resx, resy, CV_8U);
    for(int i=0; i<resx; i++)
    {
        uchar *row= curr_patch.ptr(i);
        for(int j=0; j<resy; j++)
        {
            row[j]= curr_pix_vals_temp(i*resx+j);
        }
    }
    
    curr_fmaps= extract_features(curr_patch, layer_name);
    int c=0;
    int c1=0;
    for (int m=1; m<nfmaps+1; m++)	
    {
        c1=0;
        for(int i=0; i<curr_fmaps[m].rows; i++)
        {
            float *row= curr_fmaps[m].ptr<float>(i);
            for( int j=0; j<curr_fmaps[m].cols; j++)
            {
                temp_fmaps(c1)= row[j];
                c1++;
            }
        
        }
        
        if(e_zncc) 
        {
            curr_pix_mean = temp_fmaps.mean();
        	temp_fmaps = (temp_fmaps.array() - curr_pix_mean);
            curr_pix_var = temp_fmaps.squaredNorm() / (curr_fmaps[0].rows*curr_fmaps[0].cols);
            curr_pix_std = sqrt(curr_pix_var);
            temp_fmaps /= curr_pix_std;
            
            //pix_norm_mult = 1.0 / curr_pix_std;
            //pix_norm_add = curr_pix_mean;
        }   
        c1=0;
        for(int i=0; i<curr_fmaps[m].rows; i++)
        {
            for( int j=0; j<curr_fmaps[m].cols; j++)
            {
                curr_pix_vals(c)= temp_fmaps(c1);
                c++;
                c1++;
            }
        }
        c1=0;
    }
}

FMaps::FMaps(ParamType *img_params) :
    SSDBase(img_params),params(img_params)
{

    nfmaps= params.nfmaps;
    std::cout<<"Layer name is contructor "<<params.layer_name<<std::endl;
    layer_name= params.layer_name;
    vis= params.vis;
    e_zncc= params.e_zncc;
    n_pix= nfmaps*54*54;

    const string& model_file= "../../VGG_Models/VGG_deploy.prototxt";
    const string& trained_file= "../../VGG_Models/VGG_CNN_F.caffemodel";
    const string& mean_file= "../../VGG_Models/VGG_mean.binaryproto";

    #ifdef CPU_ONLY
      Caffe::set_mode(Caffe::CPU);
    #else
      Caffe::set_mode(Caffe::GPU);
    #endif

    /* Load the network. */
    net_.reset(new Net<float>(model_file, TEST));
    net_->CopyTrainedLayersFrom(trained_file);

    CHECK_EQ(net_->num_inputs(), 1) << "Network should have exactly one input.";
    CHECK_EQ(net_->num_outputs(), 1) << "Network should have exactly one output.";

    Blob<float>* input_layer = net_->input_blobs()[0];
    num_channels_ = input_layer->channels();
    CHECK(num_channels_ == 3 || num_channels_ == 1)
        << "Input layer should have 1 or 3 channels.";
    input_geometry_ = cv::Size(input_layer->width(), input_layer->height());

    init_pix_mean = init_pix_var = init_pix_std = 0;
    curr_pix_mean = curr_pix_var = curr_pix_std = 0;

    /* Load the binaryproto mean file. */
    set_mean(mean_file);

}

/* Load the mean file in binaryproto format. */
void FMaps::set_mean(const string& mean_file) {
  BlobProto blob_proto;
  ReadProtoFromBinaryFileOrDie(mean_file.c_str(), &blob_proto);

  /* Convert from BlobProto to Blob<float> */
  Blob<float> mean_blob;
  mean_blob.FromProto(blob_proto);
  CHECK_EQ(mean_blob.channels(), num_channels_)
    << "Number of channels of mean file doesn't match input layer.";

  /* The format of the mean file is planar 32-bit float BGR or grayscale. */
  std::vector<cv::Mat> channels;
  float* data = mean_blob.mutable_cpu_data();
  for (int i = 0; i < num_channels_; ++i) {
    /* Extract an individual channel. */
    cv::Mat channel(mean_blob.height(), mean_blob.width(), CV_32FC1, data);
    channels.push_back(channel);
    data += mean_blob.height() * mean_blob.width();
  }

  /* Merge the separate channels into a single image. */
  cv::Mat mean;
  cv::merge(channels, mean);

  /* Compute the global mean pixel value and create a mean image
   * filled with this value. */
  cv::Scalar channel_mean = cv::mean(mean);
  mean_ = cv::Mat(input_geometry_, mean.type(), channel_mean);
}

void FMaps::wrap_any_layer(std::vector<cv::Mat>* input_channels, boost::shared_ptr<Blob<float> > layer) {
  int width = layer->width();
  int height = layer->height();
  float* input_data =layer->mutable_cpu_data();
  for (int i = 0; i < layer->channels(); ++i) {
    cv::Mat channel(height, width, CV_32FC1, input_data);
    input_channels->push_back(channel);
    input_data += width * height;
  }
}

std::vector<cv::Mat> FMaps::extract_features(cv::Mat img, char* l_name)
{
  Blob<float>* input_layer = net_->input_blobs()[0];
  input_layer->Reshape(1, num_channels_,
                       input_geometry_.height, input_geometry_.width);
  /* Forward dimension change to all layers. */
  net_->Reshape();

  std::vector<cv::Mat> input_channels;
  wrap_input_layer(&input_channels);

  preprocess(img, &input_channels);

  net_->ForwardPrefilled();

  const boost::shared_ptr< Blob<float> > feature_blob= net_->blob_by_name(l_name);
  //std::cout<<"Extracted feature map from conv2 , size: ";
  //vector<int> v= feature_blob->shape();
  //for(int i=0; i<v.size(); i++)
    //cout<<v[i]<<" ";
  //cout<<endl;
  //cout<<"W H "<<feature_blob->width()<<" "<<feature_blob->height()<<endl;
  std::vector<Mat> fmap;
  wrap_any_layer(&fmap,feature_blob); 
  if (vis)
  {
     for (int i=0; i<fmap.size(); i++)
     {
       cv::Mat mp= convert_float_img(fmap[i]);
       std::cout<<"Showing feature Map "<<i<<endl;
       imshow("Feature Map", mp);
       waitKey();
     }
  }
  return fmap; 
}

/* Wrap the input layer of the network in separate cv::Mat objects
 * (one per channel). This way we save one memcpy operation and we
 * don't need to rely on cudaMemcpy2D. The last preprocessing
 * operation will write the separate channels directly to the input
 * layer. */
void FMaps::wrap_input_layer(std::vector<cv::Mat>* input_channels) {
  Blob<float>* input_layer = net_->input_blobs()[0];

  int width = input_layer->width();
  int height = input_layer->height();
  float* input_data = input_layer->mutable_cpu_data();
  for (int i = 0; i < input_layer->channels(); ++i) {
    cv::Mat channel(height, width, CV_32FC1, input_data);
    input_channels->push_back(channel);
    input_data += width * height;
  }
}

void FMaps::preprocess(const cv::Mat& img,std::vector<cv::Mat>* input_channels) {
  /* Convert the input image to the input image format of the network. */
  cv::Mat sample;
  if (img.channels() == 3 && num_channels_ == 1)
    cv::cvtColor(img, sample, CV_BGR2GRAY);
  else if (img.channels() == 4 && num_channels_ == 1)
    cv::cvtColor(img, sample, CV_BGRA2GRAY);
  else if (img.channels() == 4 && num_channels_ == 3)
    cv::cvtColor(img, sample, CV_BGRA2BGR);
  else if (img.channels() == 1 && num_channels_ == 3)
    cv::cvtColor(img, sample, CV_GRAY2BGR);
  else
    sample = img;

  cv::Mat sample_resized;
  if (sample.size() != input_geometry_)
    cv::resize(sample, sample_resized, input_geometry_);
  else
    sample_resized = sample;

  cv::Mat sample_float;
  if (num_channels_ == 3)
    sample_resized.convertTo(sample_float, CV_32FC3);
  else
    sample_resized.convertTo(sample_float, CV_32FC1);

  cv::Mat sample_normalized;
  cv::subtract(sample_float, mean_, sample_normalized);

  /* This operation will write the separate BGR planes directly to the
   * input layer of the network because it is wrapped by the cv::Mat
   * objects in input_channels. */
  cv::split(sample_normalized, *input_channels);

 // CHECK(reinterpret_cast<float*>(input_channels->at(0).data)
   //     == net_->input_blobs()[0]->cpu_data())
   // << "Input channels are not wrapping the input layer of the network.";
}

//int main(int argc, char** argv) {
//  if (argc != 5) {
//    std::cerr << "Usage: " << argv[0]
//              << " deploy.prototxt network.caffemodel"
//              << " mean.binaryproto labels.txt img.jpg" << std::endl;
//    return 1;
//  }
//
//  ::google::InitGoogleLogging(argv[0]);
//
//  string model_file   = argv[1];
//  string trained_file = argv[2];
//  string mean_file    = argv[3];
//  FExtract fextract(model_file, trained_file, mean_file);
//
//  string file = argv[4];
//
//  cv::Mat img = cv::imread(file, -1);
//  CHECK(!img.empty()) << "Unable to decode image " << file;
//  fextract.extract_features(img, "conv1");
//
//}

_MTF_END_NAMESPACE
