#include "mtf/AM/PCA.h"
//#include "mtf/AM/ImageBase.h"
#include <fstream>      // std::ifstream
#include <iostream>     // std::cout
#include "opencv2/highgui/highgui.hpp"
_MTF_BEGIN_NAMESPACE

PCA::PCA(ParamType *pca_params) :
SSDBase(pca_params){
	printf("\n");
	printf("Initializing PCA appearance model with...\n");
	printf("grad_eps: %e\n", grad_eps);
	printf("hess_eps: %e\n", hess_eps);
	name = "pca";
	n_feat = 10;
	// ****** Read basis patches from files
	printf("Now reading the training patches...\n");
	string in_fname = "TMT_nl_mugII_s3_2.bin"; // hard-coded...
	ifstream in_file;
	// Define Eigen Matrix to hold the x frames by 10000 matrix
	in_file.open(in_fname, ios::in | ios::binary);
	if(in_file.is_open()) {
		printf("Now reading the training patches...\n");
	    in_file.seekg (0, in_file.end);
	    int length = in_file.tellg();
	    in_file.seekg (0, in_file.beg);
	    int num_elements = length/sizeof(double);
	    std::cout << "Reading " << num_elements << " numbers ... " << std::endl;
	    int cols = 2500;
	    int rows = num_elements/cols;
	    std::cout << "Number of rows: " << rows << std::endl;
		MatrixXd extracted_patches(rows, cols);
	    // read data as a block:
		in_file.read((char*)(extracted_patches.data()), length);
		in_file.close();

		// Print the data in matrix
		std::cout << "The extracted patches is: " << extracted_patches.col(0)
				<< std::endl;

		// ****** Compute PCA
		// Mean center the data
		printf("PCA: start of computation.\n");
		RowVectorXd meanPixel = extracted_patches.colwise().mean();
		MatrixXd centered = extracted_patches.rowwise() - meanPixel;
		// Transpose to make each image as column vector
		centered.transposeInPlace();

		// Compute the sample covariance matrix.
		MatrixXd cov = centered.transpose() * centered;
		cov = cov / (centered.cols() - 1);

		// Eigen Decomposition
		SelfAdjointEigenSolver<MatrixXd> eig(cov);
		if (eig.info() != Success) abort();
		// Get the major eigenvectors and omit the others.
		cout << "The eigenvalues are:\n" << eig.eigenvalues() << endl;
		MatrixXd evecs = eig.eigenvectors();
		MatrixXd pcaTransform = evecs.rightCols(n_feat);
		cout << "The eigen vectors are:\n" << evecs << endl;

		// Map the template in the new eigen space
		eigen_patch = centered * pcaTransform;
		cout << "The eigen patch is: \n" << eigen_patch.row(1) << endl;
		// Show the eigen_patch as image
		//for (int i=0; i < eigen_patch.cols();i++) {
		//	// Construct Mat object for current patch. It points to the memory without copying data
		//	cv::Mat curr_img_cv = cv::Mat(getResY(), getResX(), CV_64FC1,
		//		const_cast<double*>(eigen_patch.col(i).data()));
		//	// Construct Mat object for the uint8 gray scale patch
		//	cv::Mat curr_img_cv_uchar(getResY(), getResX(), CV_8UC1);
		//	// Convert the RGB patch to grayscale
		//	curr_img_cv.convertTo(curr_img_cv_uchar, curr_img_cv_uchar.type());
		//	imshow("Patch", curr_img_cv_uchar);
		//	if (cv::waitKey(1) == 27)
		//		break;
		//}


		// Show the condition numbers of basis
		JacobiSVD<MatrixXd> svd(pcaTransform);
		double cond = svd.singularValues()(0) / svd.singularValues()(svd.singularValues().size()-1);
		cout << "Condition number is: " << cond << endl;
		printf("PCA: end of computation.\n");

	} else {
		printf("PCA Could not open binary file.\n");
	}

}

	void PCA::updateDistFeat(){
		eigen_feat = eigen_patch.transpose() * getCurrPixVals();
		std::cout << "The eigen feature is " << eigen_feat << std::endl;
	}

	const double* PCA::getDistFeat(){
		return getCurrFeat().data();
	}

	double PCA::getLikelihood(){
		// Use the metric from SSDBase, but normalize over number of features instead of
		// the number of pixels
		return exp(-sqrt(-similarity/static_cast<double>(n_feat)));
	}

	void PCA::initialize() {
		 SSDBase::initialize();
		 printf("eigen_patch size: %ld, %ld\n", eigen_patch.rows(), eigen_patch.cols());
		 init_feat = eigen_patch.transpose() * getInitPixVals();
		 std::cout << "The init eigen feature is " << init_feat << std::endl;
	}

	void PCA::update(bool prereq_only/*=true*/) {
		curr_feat = eigen_patch.transpose() * getCurrPixVals();
		std::cout << "The current eigen feature is " << curr_feat.transpose() << std::endl;
		curr_feat_diff = curr_feat - init_feat;
		std::cout << "The current feature diff is " << curr_feat_diff.transpose() << std::endl;
		if(prereq_only){ return; }
#ifndef DISABLE_SPI
		if(spi_mask){
			VectorXd masked_err_vec = VectorXbM((bool*)spi_mask, n_feat).select(curr_feat, 0);
			similarity = -masked_err_vec.squaredNorm() / 2;
		} else{
#endif
			similarity = -curr_feat_diff.squaredNorm() / 2;
#ifndef DISABLE_SPI
		}
#endif
		std::cout << "The similarity is " << similarity  << std::endl;
	}

/*
void PCA::initializePixVals(const Matrix2Xd& init_pts) {
	// Call the function from the base class
	ImageBase::initializePixVals(int_pts);
}*/


/*void ImageBase::initializePixVals(const Matrix2Xd& init_pts){
	if(!isInitialized()->pix_vals){
		init_pix_vals.resize(n_pix);
		curr_pix_vals.resize(n_pix);
	}
	++frame_count;
	utils::getPixVals(init_pix_vals, curr_img, init_pts, n_pix,
		img_height, img_width, pix_norm_mult, pix_norm_add);

	if(!isInitialized()->pix_vals){
		curr_pix_vals = init_pix_vals;
		isInitialized()->pix_vals = true;
	}
}
*/
_MTF_END_NAMESPACE

