# ------------------------------------------------------------------------------------ #
# ---------------------------------- Search Methods ---------------------------------- #
# ------------------------------------------------------------------------------------ #
SM_INCLUDE_DIR = SM/include
SM_SRC_DIR = SM/src
SM_HEADER_DIR = ${SM_INCLUDE_DIR}/mtf/SM
SM_BASE_HEADERS =  ${SM_HEADER_DIR}/SearchMethod.h
SM_NT_BASE_HEADERS =  ${SM_HEADER_DIR}/NT/SearchMethod.h

MTF_INCLUDE_DIRS += ${SM_INCLUDE_DIR}

BASE_HEADERS += ${SM_HEADER_DIR}/SearchMethod.h
SEARCH_METHODS = FCLK PF GNN ESM AESM ICLK FALK IALK FCGD
SEARCH_METHODS_NT = FCLK ICLK FALK IALK ESM

COMPOSITE =  GridTracker GridTrackerCV  RKLT CascadeTracker CascadeSM ParallelTracker ParallelSM
SEARCH_OBJS = $(addprefix ${BUILD_DIR}/,$(addsuffix .o, ${SEARCH_METHODS}))
SEARCH_NT_OBJS = $(addprefix ${BUILD_DIR}/,$(addsuffix _NT.o, ${SEARCH_METHODS_NT}))
COMPOSITE_OBJS = $(addprefix ${BUILD_DIR}/,$(addsuffix .o, ${COMPOSITE}))
SEARCH_HEADERS = $(addprefix ${SM_HEADER_DIR}/, $(addsuffix .h, ${SEARCH_METHODS}))
SEARCH_NT_HEADERS = $(addprefix ${SM_HEADER_DIR}/NT/, $(addsuffix .h, ${SEARCH_METHODS_NT}))
COMPOSITE_HEADERS = $(addprefix ${SM_HEADER_DIR}/, $(addsuffix .h, ${COMPOSITE}))

SEARCH_HEADERS += ${SM_BASE_HEADERS}
SEARCH_NT_HEADERS += ${SM_NT_BASE_HEADERS}

MTF_HEADERS += ${SEARCH_HEADERS} ${COMPOSITE_HEADERS} ${SEARCH_NT_HEADERS}
MTF_OBJS += ${SEARCH_OBJS} ${COMPOSITE_OBJS} ${SEARCH_NT_OBJS}

GRID_FLAGS = 
PRL_FLAGS = 
ESM_FLAGS = 
IC_FLAGS =
FC_FLAGS = 
FA_FLAGS = 
NN_FLAGS = 

el ?= 0
et ?= 0
ed ?= 0
emg ?= 1
icd ?= 0
icl ?= 0
ict ?= 0
fcd ?= 0
fct ?= 0
fad ?= 0
fat ?= 0
iat ?= 0
nn ?= 1
prltbb ?= 0
prlomp ?= 0
grtbb ?= 1
efd ?= 0

ifeq (${nn}, 1)
SEARCH_METHODS += NN
MTF_LIBS += -lflann -lhdf5
else
MTF_RUNTIME_FLAGS += -D DISABLE_NN
endif

ifeq (${prltbb}, 1)
PRL_FLAGS += -D ENABLE_TBB
MTF_RUNTIME_FLAGS += -D ENABLE_PARALLEL
endif

ifeq (${grtbb}, 1)
GRID_FLAGS += -D ENABLE_TBB
MTF_RUNTIME_FLAGS += -D ENABLE_PARALLEL
endif

ifeq (${et}, 1)
ESM_FLAGS += -D ENABLE_PROFILING
endif
ifeq (${ed}, 1)
ESM_FLAGS += -D LOG_ESM_DATA
endif
ifeq (${el}, 1)
ESM_FLAGS += -D LOG_ESM_DATA -D ENABLE_PROFILING
endif
ifeq (${emg}, 0)
ESM_FLAGS += -D DISABLE_MEAN_GRADIENT
endif

ifeq (${icd}, 1)
IC_FLAGS += -D LOG_ICLK_DATA
endif
ifeq (${ict}, 1)
IC_FLAGS += -D LOG_ICLK_TIMES 
endif

ifeq (${fcd}, 1)
FC_FLAGS += -D LOG_FCLK_DATA
endif
ifeq (${fct}, 1)
FC_FLAGS += -D ENABLE_PROFILING 
endif

ifeq (${fad}, 1)
FA_FLAGS += -D LOG_FALK_DATA
endif
ifeq (${fat}, 1)
FA_FLAGS += -D LOG_FALK_TIMES 
endif
ifeq (${iat}, 1)
IA_FLAGS += -D LOG_IALK_TIMES 
endif
ifeq (${nnt}, 1)
NN_FLAGS += -D ENABLE_PROFILING
endif
	
${BUILD_DIR}/HACLK.o: ${SM_SRC_DIR}/HACLK.cc ${SM_HEADER_DIR}/HACLK.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${HAC_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/ICLK.o: ${SM_SRC_DIR}/ICLK.cc ${SM_HEADER_DIR}/ICLK.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${IC_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/FCLK.o: ${SM_SRC_DIR}/FCLK.cc ${SM_HEADER_DIR}/FCLK.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${FC_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/FALK.o: ${SM_SRC_DIR}/FALK.cc ${SM_HEADER_DIR}/FALK.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${FA_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/IALK.o: ${SM_SRC_DIR}/IALK.cc ${SM_HEADER_DIR}/IALK.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${IA_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/IALK2.o: ${SM_SRC_DIR}/IALK2.cc ${SM_HEADER_DIR}/IALK2.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${IA_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/FCGD.o: ${SM_SRC_DIR}/FCGD.cc ${SM_HEADER_DIR}/FCGD.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${FC_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/HESM.o: ${SM_SRC_DIR}/HESM.cc ${SM_HEADER_DIR}/HESM.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
		${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${FC_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@

${BUILD_DIR}/AESM.o: ${SM_SRC_DIR}/AESM.cc ${SM_HEADER_DIR}/AESM.h ${SM_HEADER_DIR}/ESM.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${ESM_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@

${BUILD_DIR}/ESM.o: ${SM_SRC_DIR}/ESM.cc ${SM_HEADER_DIR}/ESM.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${ESM_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/FESMBase.o: ${SM_SRC_DIR}/FESMBase.cc ${SM_HEADER_DIR}/FESMBase.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${FESMBase_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@	
	
${BUILD_DIR}/FESM.o: ${SM_SRC_DIR}/FESM.cc ${SM_HEADER_DIR}/FESM.h ${SM_HEADER_DIR}/FESMBase.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${FESM_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@	
	
${BUILD_DIR}/PF.o: ${SM_SRC_DIR}/PF.cc ${SM_HEADER_DIR}/PF.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${PF_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/NN.o: ${SM_SRC_DIR}/NN.cc ${SM_HEADER_DIR}/NN.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${NN_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/GNN.o: ${SM_SRC_DIR}/GNN.cc ${SM_HEADER_DIR}/GNN.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h 
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${GNN_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@	
${MTF_LIB_INSTALL_DIR}/libgnn.so: ${SM_HEADER_DIR}/GNN/libgnn.so
	cp -f $< $@
${SM_HEADER_DIR}/GNN/libgnn.so: ${SM_HEADER_DIR}/GNN/build_graph.h ${SM_SRC_DIR}/GNN/build_graph.cc ${SM_HEADER_DIR}/GNN/search_graph_knn.h ${SM_SRC_DIR}/GNN/search_graph_knn.cc ${SM_SRC_DIR}/GNN/utility.cc ${SM_HEADER_DIR}/GNN/utility.h ${SM_SRC_DIR}/GNN/memwatch.cc ${SM_HEADER_DIR}/GNN/memwatch.h 
	$(MAKE) -C ${SM_HEADER_DIR}/GNN --no-print-directory
	
# ------------------------------------------------------------------------------- #
# ---------------------------------- Composite ---------------------------------- #
# ------------------------------------------------------------------------------- #

${BUILD_DIR}/CascadeTracker.o: ${SM_SRC_DIR}/CascadeTracker.cc ${SM_HEADER_DIR}/CascadeTracker.h ${ROOT_HEADER_DIR}/TrackerBase.h  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
${BUILD_DIR}/CascadeSM.o: ${SM_SRC_DIR}/CascadeSM.cc ${SM_HEADER_DIR}/CascadeSM.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/ParallelTracker.o: ${SM_SRC_DIR}/ParallelTracker.cc ${SM_HEADER_DIR}/ParallelTracker.h ${ROOT_HEADER_DIR}/TrackerBase.h ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${PRL_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
${BUILD_DIR}/ParallelSM.o: ${SM_SRC_DIR}/ParallelSM.cc ${SM_HEADER_DIR}/ParallelSM.h ${APPEARANCE_HEADERS} ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${PRL_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@	
	
${BUILD_DIR}/GridTrackerCV.o: ${SM_SRC_DIR}/GridTrackerCV.cc ${SM_HEADER_DIR}/GridTrackerCV.h ${SM_HEADER_DIR}/GridBase.h ${MACROS_HEADER_DIR}/common.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@	
	
${BUILD_DIR}/RKLT.o: ${SM_SRC_DIR}/RKLT.cc ${SM_HEADER_DIR}/RKLT.h ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h ${SM_HEADER_DIR}/GridBase.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${GRID_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/GridTracker.o: ${SM_SRC_DIR}/GridTracker.cc ${SM_HEADER_DIR}/GridTracker.h ${STATE_SPACE_HEADERS} ${SM_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h ${MACROS_HEADER_DIR}/register.h ${SM_HEADER_DIR}/GridBase.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${GRID_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
# ----------------------------------------------------------------------------------- #
# ---------------------------------- Non Templated ---------------------------------- #
# ----------------------------------------------------------------------------------- #
	
${BUILD_DIR}/FCLK_NT.o: ${SM_SRC_DIR}/NT/FCLK.cc ${SM_HEADER_DIR}/NT/FCLK.h ${AM_BASE_HEADERS} ${SSM_BASE_HEADERS} ${SM_NT_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h 
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${FC_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/ICLK_NT.o: ${SM_SRC_DIR}/NT/ICLK.cc ${SM_HEADER_DIR}/NT/ICLK.h ${AM_BASE_HEADERS} ${SSM_BASE_HEADERS} ${SM_NT_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h 
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${IC_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/FALK_NT.o: ${SM_SRC_DIR}/NT/FALK.cc ${SM_HEADER_DIR}/NT/FALK.h ${AM_BASE_HEADERS} ${SSM_BASE_HEADERS} ${SM_NT_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h 
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${FA_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/IALK_NT.o: ${SM_SRC_DIR}/NT/IALK.cc ${SM_HEADER_DIR}/NT/IALK.h ${AM_BASE_HEADERS} ${SSM_BASE_HEADERS} ${SM_NT_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${IA_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
	
${BUILD_DIR}/ESM_NT.o: ${SM_SRC_DIR}/NT/ESM.cc ${SM_HEADER_DIR}/NT/ESM.h ${AM_BASE_HEADERS} ${SSM_BASE_HEADERS} ${SM_NT_BASE_HEADERS}  ${UTILITIES_HEADER_DIR}/miscUtils.h ${MACROS_HEADER_DIR}/common.h 
	${CXX} -c -fPIC ${WARNING_FLAGS} ${OPT_FLAGS} ${PROF_FLAGS} ${CT_FLAGS} ${MTF_INCLUDE_FLAGS} ${ESM_FLAGS} $< ${FLAGS64} ${FLAGSCV} -o $@
